function varargout=klm(flag,varargin)
% *********************************************************************
% Implements a (time varying) Kalman filter.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=klm(3,pred,yt,ut) % off line version
%                             klm(3,pred,yt)    % off line version
%        [xh,P]=klm(0,pred)        % Initialisering recursive version
%        [xh,P,res,R,K]=klm(1,y)   % Data update
%        [xh,P]=klm(2,u)           % Time update
%        [xh,P]=klm(2)             % Time update
% -------------------------------------------------------------------------
% yt    data sequence of measurements (outputs)
% ut    data sequence of inputs
% y	    measurement
% u	    input
% pred  predictive (1) or ordinary (0) version
% *************************************************************************
% Private notes:
% klm så simpel som mulig.
% u påvirker kun tidsopdatering.
% data opsamling sker for flag=3 (off line version)
% recest deles i 2 (tid og data update) og sender kun xh ud
%    Data opsamles i recest.
% *************************************************************************

persistent pred
persistent nx nu nv ny ne
persistent x0 P0 vm R1 em R2
persistent A B C D Gx Gy
persistent xh P 

switch flag,
 % *********************************************************************
 case 0, % Initialisering
  if nargin>1, pred=varargin{1}; else, pred=0; end;
  [stype,nx,nu,nv,ny,ne]=initt(3);
  [x0,P0,vm,R1,em,R2]=initt(4);
  xh=x0; P=P0;
  [A,B,Gx]=statelin; % LTI system
  [C,D,Gy]=measlin;
  varargout={xh, P};
 % ------------------------------------------------------------------------
 % Data update
 % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  yh=C*xh+em;
  R=C*P*C'+R2;
  K=P*C'*inv(R);
  res=y-yh;
  if ~misobs(y),
   xh=xh+K*res;
   P=P-K*C*P;
  end;
  varargout={xh,P,res,yh,R,K};
 % -----------------------------------------------------------------------
 % Time update
 % ------------------------------------------------------------------------
 case 2
  if nu>0, u=varargin{1}; else, u=[]; end;
  if isempty(Gx),
   if nu>0, xh=A*xh+B*u+vm; else, xh=A*xh+vm; end;
   P=A*P*A'+R1;
  else
   if nu>0, xh=A*xh+B*u+Gx*vm; else, xh=A*xh+Gx*vm; end;
   P=A*P*A'+Gx*R1*Gx';
  end;
  varargout={xh,P};
 % *********************************************************************
 otherwise
 % *********************************************************************
end % flag end

