function varargout=skfb(flag,varargin)
% *********************************************************************
% Implements a simple particle Kalman filter. Based on Bayes theorem.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=skfb(3,pred,yt,ut) % off line version
%                             skfb(3,pred,yt)    % off line version
%        skfb(0,pred)               % Initialisering recursive version
%        [xh,P,res,R,K]=skfb(1,y)   % Data update
%        [xh,P]=skfb(2,u)           % Time update
%        [xh,P]=skfb(2)             % Time update
% -------------------------------------------------------------------------
% yt    data sequence of measurements (outputs)
% ut    data sequence of inputs
% y	    measurement
% u	    input
% pred  predictive (1) or ordinary (0) version
% *************************************************************************

persistent pred
persistent nx nu nv ny ne
persistent x0 P0 vm R1 em R2
persistent xh P
persistent np Sv Se w fak R2inv

switch flag,
 % *********************************************************************
 case 0, % Initialisering
  if nargin>1, pred=varargin{1}; else, pred=0; end;
  [stype,nx,nu,nv,ny,ne]=initt(3);
  [x0,P0,vm,R1,em,R2]=initt(4);
  R2inv=inv(R2);
  Sv=msqrt(R1); Se=msqrt(R2);
  xh=x0; P=P0;
  np=200;
  w=zeros(1,np);
  fak=1/2/pi/sqrt(det(R2));
  varargout={xh, P};
  % ------------------------------------------------------------------------
  % Data update
  % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  Sx=msqrt(P);
  Dx=Sx*randn(nx,np);
  X=kron(ones(1,np),xh)+Dx;
  E=kron(ones(1,np),em)+Se*randn(ne,np);
  Y=measmodel([X; E]);
  res=kron(ones(1,np),y)-Y;

  if ~misobs(y),
   for j=1:np,
    err=res(:,j);
    w(j)=fak*exp(-0.5*err'*R2inv*err);
   end
   w=w/sum(w');

   xh=kron(ones(nx,1),w).*X;
   xh=sum(xh')';
   yh=kron(ones(ny,1),w).*Y;
   yh=sum(yh')';
   res=kron(ones(ny,1),w).*res;
   res=sum(res')';

   Dx=X-kron(ones(1,size(X,2)),xh);
   P=Dx*diag(w)*Dx';
  else
   yh=NaN(ny,1); res=NaN(ny,1);
  end;

  R=[]; K=[];
  varargout={xh,P,res,yh,R,K};
  % ---------------------------------------------------------------------
  % Time update
  % ---------------------------------------------------------------------
 case 2,
  if nu>0, u=varargin{1}; else, u=[]; end;
  Sx=msqrt(P);
  X=kron(ones(1,np),xh)+Sx*randn(nx,np);
  V=kron(ones(1,np),vm)+Sv*randn(nv,np);
  if nu>0, U=kron(ones(1,np),u); else, U=[]; end;

  X=statemodel([X; U; V]);
  xh=mean(X')';
  Dx=X-kron(ones(1,size(X,2)),xh);
  P=Dx*Dx'/np;

  varargout={xh,P};
  % *********************************************************************
 otherwise
  % *********************************************************************
end % flag end

