function varargout=skfp(flag,varargin)
% *********************************************************************
% Implements a simple particle Kalman filter. Projection based.
% ---------------------------------------------------------------------
% Usage: [xht,St,yht,rest,Rt]=skfp(3,pred,yt,ut) % off line version
%                             skfp(3,pred,yt)    % off line version
%        skfp(0,pred)               % Initialisering recursive version
%        [xh,P,res,R,K]=skfp(1,y)   % Data update
%        [xh,P]=skfp(2,u)           % Time update
%        [xh,P]=skfp(2)             % Time update
% -------------------------------------------------------------------------
% yt    data sequence of measurements (outputs)
% ut    data sequence of inputs
% y	    measurement
% u	    input
% pred  predictive (1) or ordinary (0) version
% *************************************************************************

persistent pred
persistent nx nu nv ny ne
persistent x0 P0 vm R1 em R2
persistent xh P
persistent np Sv Se

switch flag,
 % *********************************************************************
 case 0, % Initialisering
  if nargin>1, pred=varargin{1}; else, pred=0; end;
  [stype,nx,nu,nv,ny,ne]=initt(3);
  [x0,P0,vm,R1,em,R2]=initt(4);
  Sv=msqrt(R1); Se=msqrt(R2);
  xh=x0; P=P0;
  np=200;
  varargout={xh, P};
  % ------------------------------------------------------------------------
  % Data update
  % ------------------------------------------------------------------------
 case 1,
  y=varargin{1};
  Sx=msqrt(P);
  Dx=Sx*randn(nx,np);
  X=kron(ones(1,np),xh)+Dx;
  E=kron(ones(1,np),em)+Se*randn(ne,np);
  Y=measmodel([X; E]);
  yh=mean(Y')';
  Dy=Y-kron(ones(1,size(Y,2)),yh);
  R=Dy*Dy'/np;
  Pxy=Dx*Dy'/np;

  K=Pxy*inv(R);
  res=y-yh;
  if ~misobs(y),
   xh=xh+K*res;
   P=P-K*Pxy';
   % to avoid negative defnite matrices.
   while min(eig(P))<0, P=P+0.1*K*Pxy; disp('c'); end;
  end;

  varargout={xh,P,res,yh,R,K};
  % -----------------------------------------------------------------------
  % Time update
  % ------------------------------------------------------------------------
 case 2
  if nu>0, u=varargin{1}; else, u=[]; end;
  Sx=msqrt(P);
  X=kron(ones(1,np),xh)+Sx*randn(nx,np);
  V=kron(ones(1,np),vm)+Sv*randn(nv,np);
  if nu>0, U=kron(ones(1,np),u); else, U=[]; end;

  X=statemodel([X; U; V]);
  xh=mean(X')';
  Dx=X-kron(ones(1,size(X,2)),xh);
  P=Dx*Dx'/np;

  varargout={xh,P};
  % *********************************************************************
 otherwise
  % *********************************************************************
end % flag end

