function [yt,xt,vt,et]=klmsim(varargin)
% Usage: [yt,xt,vt,et]=klmsim(ut) % eXogeneuos version
%        [yt,xt,vt,et]=klmsim(Ns) % Autonomuous version
%
% ddd (deterministic, discrete, discrete)
% sdd (stochastic, discrete, discrete)
% dcd (deterministic, continuous, discrete)
% scd (stochastic, continuous, discrete)

autonom=length(varargin{1});
initt(0);
stype=initt(1);

if autonom==1,
 disp('auto')
 Ns=varargin{1};
 switch stype,
  case 'sdd', [yt,xt,vt,et]=sddklmsim(Ns); 
  % case 'scd', [yt,xt,vt,et]=scdklmsim(Ns); % not finish
  % case 'ddd', [yt,xt,vt,et]=dddklmsim(Ns); % not finish
  % case 'dcd', [yt,xt,vt,et]=dcdklmsim(Ns); % not finish
  otherwise, disp('stype out of range'); pause;
 end 
else % autonom
 ut=varargin{1};
 switch stype,
  case 'sdd', [yt,xt,vt,et]=sddxklmsim(ut);
  case 'scd', [yt,xt,vt,et]=scdxklmsim(ut);
  case 'ddd', [yt,xt,vt,et]=dddxklmsim(ut);
  case 'dcd', [yt,xt,vt,et]=dcdxklmsim(ut); 
  otherwise, disp('stype out of range'); pause;
 end
end
% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

% **********************************************************************
function [yt,xt,vt,et]=sddxklmsim(ut)
% **********************************************************************
% Stochastic, Discrete time process and discrete time observations
% ----------------------------------------------------------------------
fprintf('System - ')
[stype,nx,nu,nv,ny,ne]=initt(1);
[x0,P0,vm,R1,em,R2]=initt(2);
Se=msqrt(R2); Sv=msqrt(R1);

[nr,ns]=size(ut); if nr>ns, ut=ut'; end;
Ns=length(ut);
et=Se*randn(ne,Ns)+kron(em,ones(1,Ns)); 
vt=Sv*randn(nv,Ns)+kron(vm,ones(1,Ns)); 
yt=zeros(ny,Ns); xt=zeros(nx,Ns);
fprintf('Noise - ');

x=x0+msqrt(P0)*randn(nx,1);

for i=1:Ns,
 xt(:,i)=x;
 u=ut(:,i);

 e=et(:,i);
 y=measproc([x; e]);
 yt(:,i)=y;

 v=vt(:,i);
 x=stateproc([x; u; v]); 

end
fprintf('Data - Generated \n');
% ----------------------------------------------------------------------

% **********************************************************************
function [yt,xt,vt,et]=scdxklmsim(ut)
% **********************************************************************
% Stochastic, Continuous time process and discrete time observations
% ----------------------------------------------------------------------
ms=10;

fprintf('System - ')
[stype,nx,nu,nv,ny,ne,Ts]=initt(1);
[x0,P0,vm,R1,em,R2]=initt(2);
Tsub=Ts/ms;
Se=msqrt(R2); Sv=msqrt(R1)/sqrt(Tsub);

[nr,ns]=size(ut); if nr>ns, ut=ut'; end;
Ns=length(ut);
et=Se*randn(ne,Ns)+kron(em,ones(1,Ns)); 
vt=Sv*randn(nv,Ns*ms)+kron(vm,ones(1,Ns*ms)); 
yt=zeros(ny,Ns); xt=zeros(nx,Ns);
fprintf('Noise - ');

x=x0+msqrt(P0)*randn(nx,1);

for i=1:Ns,

 xt(:,i)=x;
 u=ut(:,i);

 e=et(:,i);
 y=measproc([x; e]);
 yt(:,i)=y;

 for j=0:(ms-1),
  v=vt(:,i+j);
  [td,xd]=ode45(@diffeq,[0 Tsub],x,[],[x; u; v],0);
  x=xd(length(xd),:)';
 end;
 
end
fprintf('Data - Generated \n');
% ----------------------------------------------------------------------

% **********************************************************************
function [yt,xt,vt,et]=dddxklmsim(ut)
% **********************************************************************
% Deterministic, Discrete time process and Discrete time observations
% ----------------------------------------------------------------------
fprintf('System - ')
[stype,nx,nu,nv,ny,ne]=initt(1);
[x0,P0,vm,R1,em,R2]=initt(2);
Se=msqrt(R2); Sv=msqrt(R1);

[nr,ns]=size(ut); if nr>ns, ut=ut'; end;
Ns=length(ut);
et=kron(em,ones(1,Ns)); vt=kron(vm,ones(1,Ns));
yt=zeros(ny,Ns); xt=zeros(nx,Ns);
fprintf('Disturbances - ');

x=x0;

for i=1:Ns,

 xt(:,i)=x;
 u=ut(:,i);

 e=et(:,i);
 y=measproc([x; e]);
 yt(:,i)=y;

 v=vt(:,i);
 x=stateproc([x; u; v]); 

end
fprintf('Data - Generated \n');
% ----------------------------------------------------------------------

% **********************************************************************
function [yt,xt,vt,et]=dcdxklmsim(ut)
% **********************************************************************
% Deterministic, Continuous time process and Discrete time observations
% ----------------------------------------------------------------------
fprintf('System - ')
[stype,nx,nu,nv,ny,ne,Ts]=initt(1);
[x0,P0,vm,R1,em,R2]=initt(2);

[nr,ns]=size(ut); if nr>ns, ut=ut'; end;
Ns=length(ut);
vt=kron(vm,ones(1,Ns)); et=kron(em,ones(1,Ns));
yt=zeros(ny,Ns); xt=zeros(nx,Ns);
fprintf('Disturbances - ');

x=x0;

for i=1:Ns,

 xt(:,i)=x;
 u=ut(:,i);

 e=et(:,i);
 y=measproc([x; e]);
 yt(:,i)=y;

 v=vt(:,i);
 [td,xd]=ode45(@diffeq,[0 Ts],x,[],[x; u; v],0);
 x=xd(length(xd),:)';

end
fprintf('Data - Generated \n');
% ----------------------------------------------------------------------

% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Autonuous part
% ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

% **********************************************************************
function [yt,xt,vt,et]=sddklmsim(Ns)
% **********************************************************************
% Usage: [yt,xt,vt,et]=klmsim(Ns)
% ----------------------------------------------------------------------

fprintf('System - ')
initt(0);
[stype,nx,nu,nv,ny,ne]=initt(1);
[x0,P0,vm,R1,em,R2]=initt(2);
Se=msqrt(R2);
Sv=msqrt(R1);

et=Se*randn(ne,Ns); vt=Sv*randn(nv,Ns); 
yt=zeros(ny,Ns); xt=zeros(nx,Ns);
fprintf('Noise - ');

x=x0+msqrt(P0)*randn(nx,1);

for i=1:Ns,
 xt(:,i)=x;

 e=et(:,i);
 y=measproc([x; e]);
 yt(:,i)=y;

 v=vt(:,i);
 x=stateproc([x; v]);


end
fprintf('Data - Generated \n');
% ----------------------------------------------------------------------
